#if !defined(LEXER_HH)
#define LEXER_HH

#include <iostream>
#include <string>

using std::string;

#include "Token.hh"
#include "ErrorManager.hh"

///////////////////////////////////////////////////////////////////////////
class Lexer {
///////////////////////////////////////////////////////////////////////////

public :

  // Constructor and destructor. The constructor requires:
  // a) a non-const reference to an input character stream, and
  // b) a non-const reference to the error manager
  Lexer( std::istream& is, ErrorManager& error ) : 
    inStream_(is), error_(error) { }
  ~Lexer( void ) { }

  // Method to advance to the next token
  void next() {
    char ch;
    // Skip white spaces, tabs, and newlines
    while (inStream_.get(ch) && isLayout_(ch));
    // Check EOF condition
    if (!inStream_.eof()) {
      if (isDigit_(ch)) {
	// Recognizing a number
	token_.setPartOfSpeech(Token::NUMBER);
	string tmp(1,ch);
	while (inStream_.get(ch) && isDigit_(ch)) {
	  tmp.push_back(ch);
	}
	inStream_.putback(ch);
	token_.setLexeme(tmp);
      } else if (isCharacter_(ch)) {
	// Recognizing an identifier 
	token_.setPartOfSpeech(Token::IDFIER);
	string tmp(1,ch);
	while (inStream_.get(ch) && isDigit_(ch)) {
	  tmp.push_back(ch);
	}
	inStream_.putback(ch);
	token_.setLexeme(tmp);
      } else {
	// Recognizing parentheses, operators and assignment
	string tmp(1,ch);
	token_.setLexeme(tmp);
	switch(ch) {
	case '(' : token_.setPartOfSpeech(Token::LPAR); break;
	case ')' : token_.setPartOfSpeech(Token::RPAR); break;
	case '+' : 
	case '-' : 
	case '*' : 
	case '/' : 
	  token_.setPartOfSpeech(Token::OPERATOR); break;
	case '=' :
	  token_.setPartOfSpeech(Token::ISEQ); break;
	default : 
	  token_.setPartOfSpeech(Token::NIL);
	}
      }
    } else {
      token_.setLexeme(string(""));
      token_.setPartOfSpeech(Token::EOFF);
    }
  }

  // Method to request the current token
  const Token& cur() {
    return token_;
  }

private :
  std::istream& inStream_;
  ErrorManager& error_;
  Token         token_;

  // Test method for layout characters
  bool isLayout_( char ch ) {
    switch (ch) {
    case ' ': case '\t' : case '\n': return true;
    default : return false;
    }
  }
  // Test method for digits
  bool isDigit_( char ch ) {
    return (ch >= '0' && ch <= '9');
  } 
  // Test method for alphabetical characters
  bool isCharacter_( char ch ) {
    return ((ch >= 'A' && ch <= 'Z') || (ch >= 'a' && ch <= 'z'));
  }

}; // End of class Lexer

#endif
